function [ pts ] = extract_pointwise_contour( I, n )
%EXTRACT_POINTWISE_CONTOUR Extract a 2xn array with nearly equidistant
% coordinates along the contour.

I        = padarray(double(I),[20 20]);
C        = getOuterContour(I);
pts      = contourResample(C, n);
pts      = pts - 20;

end

function pts = contourResample(C, N)
numC   = size(C,2);
diff   = C(:,[2:end])-C(:,[1:end-1]);
len    = [0 cumsum(sqrt(sum(diff.^2,1)))];
lenInv = interp1(len, 1:numC, linspace(0,len(end),N));
pts    = zeros(2,N);
pts(1,:) = interp1(1:numC, C(1,:), lenInv);
pts(2,:) = interp1(1:numC, C(2,:), lenInv);
end


function C = getOuterContour(I)
binI        = double(I>mean(unique(I)));
contourData = contourc(binI, [0.5 0.5]);
contourNum  = size(contourData, 2);
clear binI;

% extract all contours
numC = 0;
id   = 1;
while id<contourNum
  numC = numC+1;
  len  = contourData(2,id);
  id   = id + len + 1;
end
cellC = cell(numC,1);
numC = 0;
id   = 1;
while id<contourNum
  numC = numC+1;
  len  = contourData(2,id);
  cellC{numC} = contourData(:,id+(1:len));
  id   = id + len + 1;
end
clear id len contourData contourNum;

% find outer contour
outerId  = 1;
id       = 2;
while id<numC
  if testInside(cellC{id}(:,1), cellC{outerId})
  elseif testInside(cellC{outerId}(:,1), cellC{id})
    outerId = id;
  elseif contourLength(cellC{id}) > contourLength(cellC{outerId})
    outerId = id;
  end
  id = id + 1;
end
C = cellC{outerId};
end


function b=testInside(p,C)
  numC = size(C,2);
  diff  = C - repmat(p, [1 numC]);
  alpha = atan2(diff(2,:), diff(1,:));
  alphaDiff = alpha(2:end) - alpha(1:end-1);
  ind = (alphaDiff<-pi);
  alphaDiff(ind) = alphaDiff(ind)+2*pi;
  ind = (alphaDiff>pi);
  alphaDiff(ind) = alphaDiff(ind)-2*pi;
  b = abs(sum(alphaDiff))>pi;
end


function len = contourLength(C)
  diff = C(:,[2:end 1])-C;
  len  = sum(sqrt(sum(diff.^2,1)));
end

